package Kirjalista;

import java.util.ArrayList;
import tietorakenne.LinkattuLista;


/**
 * Genren ja kirjan välinen relaatio
 * - pitää kirjaa liimatuista genreistä ja kirjoista
 * - palauttaa kirjojen id:t genren id:n perusteella
 * - palauttaa genrejen id:t kirjan id:n perusteella
 * - lukee ja kirjoittaa tiedostoon
 */
public class GenreRelaatio {
    private LinkattuLista<KirjanGenre> alkiot = new LinkattuLista<KirjanGenre>();
    
    /**
     * Lisää listaan alkion
     * @param alkio Alkio, joka lisätään listaan
     */
    public void lisaa(KirjanGenre alkio) {
        alkiot.lisaa(alkio);
    }
    
    
    /**
     * Palauttaa ArrayListin kirjojen id:istä, joilla on tietty genreid
     * esim. palauttaa kaikkien kirjojen id:t, joiden genre on fantasia
     * @param genreId Genren id, jonka perusteella haetaan kirjojen id:itä
     * @return Kaikkien niiden kirjojen id:t, joiden genren id on genreId 
     * <pre name="test">
     * #import java.util.ArrayList;
     * GenreRelaatio gr = new GenreRelaatio();
     * Kirja k = Kirja.testiKirja();
     * Kirja k1 = Kirja.testiKirja();
     * Kirja k2 = Kirja.testiKirja();
     * Genre g = Genre.testiGenre();
     * Genre g2 = Genre.testiGenre(); 
     * 
	 * gr.lisaa(new KirjanGenre(k, g));
	 * gr.lisaa(new KirjanGenre(k1, g));
	 * gr.lisaa(new KirjanGenre(k2, g));
	 * gr.lisaa(new KirjanGenre(k, g2));
	 * ArrayList<Integer> al = gr.haeKirjat(g.getId());
	 * int i = al.get(0);
	 * int i1 = al.get(1);
	 * int i2 = al.get(2);
	 * i === k.getId();	  
	 * i1 === k1.getId();	 
	 * i2 === k2.getId();
	 * al.size() === 3;
	 
	 * </pre>
     */
    public ArrayList<Integer> haeKirjat(int genreId) {
        
        ArrayList<Integer> kirjojenIdt = new ArrayList<Integer>();
        
        alkiot.iter = alkiot.getEka();
        
        for (int i = 0; i < alkiot.length(); i++) {
            KirjanGenre alkio = alkiot.iter.getSisalto();
            if (alkio.getGenreId() == genreId) {
                kirjojenIdt.add(alkio.getKirjaId());
            }
            alkiot.iter = alkiot.iter.getNext();
        }
        
        return kirjojenIdt;
    }
    
    
    /**
     * Palauttaa ArrayListin genrejen id:istä, joilla on tietty kirjaid
     * eli toisin sanoen palauttaa tietyn kirjan kaikkien genrejen id:t
     * @param kirjaId Kirjan id, jonka perusteella haetaan genrejen id:itä
     * @return Kaikkien niiden genrejen id:t, joiden kirjan id on kirjaId 
     * <pre name="test">
     * GenreRelaatio gr = new GenreRelaatio();
     * Kirja k = Kirja.testiKirja();
     * Kirja k2 = Kirja.testiKirja();
     * Genre g = Genre.testiGenre();
     * Genre g1 = Genre.testiGenre();
     * Genre g2 = Genre.testiGenre(); 
     * 
     * gr.lisaa(new KirjanGenre(k, g2));
     * gr.lisaa(new KirjanGenre(k, g));
     * gr.lisaa(new KirjanGenre(k, g1));
     * gr.lisaa(new KirjanGenre(k2, g));
     * ArrayList<Integer> al = gr.haeGenret(k.getId());
     * int i = al.get(0);
     * int i1 = al.get(1);
     * int i2 = al.get(2);
     * i === g2.getId();   
     * i1 === g.getId();     
     * i2 === g1.getId();
     * 
     * al.size() === 3; 
     * </pre>
     */
    public ArrayList<Integer> haeGenret(int kirjaId) {

        ArrayList<Integer> genrejenIdt = new ArrayList<Integer>();
        
        alkiot.iter = alkiot.getEka();
        
        for (int i = 0; i < alkiot.length(); i++) {
            KirjanGenre alkio = alkiot.iter.getSisalto();
            if (alkio.getKirjaId() == kirjaId) {
                genrejenIdt.add(alkio.getGenreId());
            }
            alkiot.iter = alkiot.iter.getNext();
        }
        
        return genrejenIdt;
    }
    
    
    /**
     * Poistaa alkion annetussa indeksipaikassa
     * @param indeksi Indeksipaikka, josta alkio poistetaan
     */
    public void poista(int indeksi) {
        alkiot.poista(indeksi);
    }
}
